@@ -22,14 +22,6 @@
 #include <stdlib.h>
 #endif
 
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-
-#ifdef HAVE_SYS_WAIT_H
-#include <sys/wait.h>
-#endif
-
 #include <glib.h>
 #include <gtk/gtk.h>
 
@@ -93,7 +85,7 @@ gboolean tgh_add (gchar **files, GPid *pid)
   if (files)
     if (chdir(files[0]))
     {
-      gchar *dirname = g_dirname (files[0]);
+      gchar *dirname = g_path_get_dirname (files[0]);
       if (chdir(dirname))
       {
         g_free (dirname);
