@@ -40,16 +40,47 @@
 #include "tgh-status-dialog.h"
 #include "tgh-log-dialog.h"
 #include "tgh-branch-dialog.h"
+#include "tgh-stash-dialog.h"
+#include "tgh-blame-dialog.h"
 
 #include "tgh-common.h"
 
 static void
-create_error_dialog(GtkWindow *parent, gchar *message)
+create_error_dialog (GtkWindow *parent, gchar *message)
 {
-  GtkWidget *error;
-  error = gtk_message_dialog_new(GTK_WINDOW(parent), GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Status failed"));
-  gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(error), "%s", g_strstrip(message));
-  tgh_dialog_start(GTK_DIALOG(error), FALSE);
+  if (TGH_IS_NOTIFY_DIALOG (parent))
+  {
+    gchar **lines, **iter;
+    lines = g_strsplit_set (message, "\r\n", -1);
+
+    for (iter = lines; *iter; iter++)
+    {
+      gchar *action, *text;
+
+      if ((*iter)[1])
+      {
+        text = *iter;
+        action = strchr (text, ':');
+        if (action)
+        {
+          *action = '\0';
+          text = action+2;
+          action = *iter;
+        }
+
+        tgh_notify_dialog_add (TGH_NOTIFY_DIALOG (parent), action, text);
+      }
+    }
+
+    g_strfreev (lines);
+  }
+  else
+  {
+    GtkWidget *error;
+    error = gtk_message_dialog_new (parent?GTK_WINDOW (parent):NULL, parent?GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL:0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Failed"));
+    gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (error), "%s", g_strstrip (message));
+    tgh_dialog_start (GTK_DIALOG (error), !parent);
+  }
 }
 
 void
@@ -68,7 +99,7 @@ void
 tgh_child_exit(GPid pid, gint status, gpointer user_data)
 {
   TghErrorParser *parser = user_data;
-  if(WEXITSTATUS(status) > 1)
+  if(WEXITSTATUS(status))
   {
     if(parser->done)
       create_error_dialog(GTK_WINDOW(parser->dialog), parser->error);
@@ -352,6 +383,190 @@ tgh_branch_parser_new (GtkWidget *dialog)
   return TGH_OUTPUT_PARSER(parser);
 }
 
+typedef struct {
+  TghOutputParser parent;
+  GtkWidget *dialog;
+} TghStashListParser;
+
+static void
+stash_list_parser_func (TghStashListParser *parser, gchar *line)
+{
+  TghStashDialog *dialog = TGH_STASH_DIALOG (parser->dialog);
+  if (line)
+  {
+    gchar *stash, *branch, *desc;
+    branch = strchr (line, ':');
+    *branch++ = '\0';
+    stash = g_strstrip (line);
+    desc = strchr (branch, ':');
+    *desc++ = '\0';
+    branch = g_strstrip (branch);
+    desc = g_strstrip (desc);
+    tgh_stash_dialog_add (dialog, stash, branch, desc);
+  }
+  else
+  {
+    tgh_stash_dialog_done (dialog);
+    g_free (parser);
+  }
+}
+
+TghOutputParser*
+tgh_stash_list_parser_new (GtkWidget *dialog)
+{
+  TghStashListParser *parser = g_new (TghStashListParser,1);
+
+  TGH_OUTPUT_PARSER (parser)->parse = TGH_OUTPUT_PARSER_FUNC (stash_list_parser_func);
+
+  parser->dialog = dialog;
+
+  return TGH_OUTPUT_PARSER (parser);
+}
+
+typedef struct {
+  TghOutputParser parent;
+  GtkWidget *dialog;
+} TghStashShowParser;
+
+static void
+stash_show_parser_func (TghStashShowParser *parser, gchar *line)
+{
+  TghStashDialog *dialog = TGH_STASH_DIALOG (parser->dialog);
+  if (line)
+  {
+    gchar *ptr, *file;
+    guint insertions = strtoul (line, &ptr, 10);
+    guint deletions = strtoul (ptr, &file, 10);
+    file++;
+    file = g_strndup (file, strlen(file)-1);
+    tgh_stash_dialog_add_file (dialog, insertions, deletions, file);
+  }
+  else
+  {
+    tgh_stash_dialog_done (dialog);
+    g_free (parser);
+  }
+}
+
+TghOutputParser*
+tgh_stash_show_parser_new (GtkWidget *dialog)
+{
+  TghStashShowParser *parser = g_new (TghStashShowParser,1);
+
+  TGH_OUTPUT_PARSER (parser)->parse = TGH_OUTPUT_PARSER_FUNC (stash_show_parser_func);
+
+  parser->dialog = dialog;
+
+  return TGH_OUTPUT_PARSER (parser);
+}
+
+typedef struct {
+  TghOutputParser parent;
+  GtkWidget *dialog;
+} TghBlameParser;
+
+static void
+blame_parser_func (TghBlameParser *parser, gchar *line)
+{
+  TghBlameDialog *dialog = TGH_BLAME_DIALOG (parser->dialog);
+  if (line)
+  {
+    gchar *revision, *name, *date, *text, *ptr;
+    guint64 line_no;
+
+    name = strchr (line, '(');
+    *name++ = '\0';
+
+    revision = g_strstrip (line);
+
+    text = strchr (name, ')');
+    *text = '\0';
+    text += 2;
+    text[strlen (text)-1] = '\0';
+
+    ptr = strrchr (name, ' ');
+    line_no = g_ascii_strtoull (ptr, NULL, 10);
+
+    while (*--ptr == ' ');
+    ptr[1] = '\0';
+
+    date = strrchr (name, ' ');
+    *date = '\0';
+    ptr = strrchr (name, ' ');
+    *date = ' ';
+    *ptr = '\0';
+    date = strrchr (name, ' ');
+    *ptr = ' ';
+    *date++ = '\0';
+
+    name = g_strstrip (name);
+
+    tgh_blame_dialog_add (dialog, line_no, revision, name, date, text);
+  }
+  else
+  {
+    tgh_blame_dialog_done (dialog);
+    g_free (parser);
+  }
+}
+
+TghOutputParser*
+tgh_blame_parser_new (GtkWidget *dialog)
+{
+  TghBlameParser *parser = g_new (TghBlameParser,1);
+
+  TGH_OUTPUT_PARSER (parser)->parse = TGH_OUTPUT_PARSER_FUNC (blame_parser_func);
+
+  parser->dialog = dialog;
+
+  return TGH_OUTPUT_PARSER (parser);
+}
+
+typedef struct {
+  TghOutputParser parent;
+  GtkWidget *dialog;
+} TghCleanParser;
+
+static void
+clean_parser_func(TghNotifyParser *parser, gchar *line)
+{
+  TghNotifyDialog *dialog = TGH_NOTIFY_DIALOG(parser->dialog);
+  if(line)
+  {
+    gchar *action, *file;
+
+    action = file = line;
+    if (g_ascii_strncasecmp (line, "Would ", 6) == 0)
+      file += 6;
+
+    if (g_ascii_strncasecmp (file, "Not ", 4) == 0)
+      file += 4;
+
+    file = strchr (file, ' ');
+    *file++ = '\0';
+    file[strlen (file)-1] = '\0';
+
+    tgh_notify_dialog_add(dialog, action, file);
+  }
+  else
+  {
+    tgh_notify_dialog_done(dialog);
+    g_free(parser);
+  }
+}
+
+TghOutputParser*
+tgh_clean_parser_new (GtkWidget *dialog)
+{
+  TghCleanParser *parser = g_new(TghCleanParser,1);
+
+  TGH_OUTPUT_PARSER(parser)->parse = TGH_OUTPUT_PARSER_FUNC(clean_parser_func);
+
+  parser->dialog = dialog;
+
+  return TGH_OUTPUT_PARSER(parser);
+}
+
 gboolean
 tgh_parse_output_func(GIOChannel *source, GIOCondition condition, gpointer data)
 {
