@@ -22,6 +22,14 @@
 #include <stdlib.h>
 #endif
 
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+
+#ifdef HAVE_SYS_WAIT_H
+#include <sys/wait.h>
+#endif
+
 #include <glib.h>
 #include <gtk/gtk.h>
 
@@ -89,7 +97,7 @@ static gboolean reset_spawn (GtkWidget *dialog, gchar **files, GPid *pid)
   }
   g_free (argv);
 
-  parser = tgh_error_parser_new(GTK_WIDGET(dialog));
+  parser = tgh_error_parser_new(NULL);
 
   args = g_new(struct exit_args, 1);
   args->parser = parser;
@@ -110,7 +118,7 @@ gboolean tgh_reset (gchar **files, GPid *pid)
   if (files)
     if (chdir(files[0]))
     {
-      gchar *dirname = g_dirname (files[0]);
+      gchar *dirname = g_path_get_dirname (files[0]);
       if (chdir(dirname))
       {
         g_free (dirname);
